import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { mockNetworkExperimentalAPIResponse } from '@/utils/mockNetworkResponse';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { HttpStatusCode } from 'axios';
import { apiPath } from '@/redux/apiPath';
import { getBioEntity } from './bioEntity.thunks';
import bioEntityContentsReducer from './bioEntity.slice';
import { BioEntityContentsState } from './bioEntity.types';

const mockedAxiosClient = mockNetworkExperimentalAPIResponse();
const SEARCH_QUERY = 'park7';

const INITIAL_STATE: BioEntityContentsState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('bioEntity reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BioEntityContentsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('bioEntity', bioEntityContentsReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(bioEntityContentsReducer(undefined, action)).toEqual(INITIAL_STATE);
  });
  it('should update store after succesfull getBioEntity query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

    const { type } = await store.dispatch(getBioEntity(SEARCH_QUERY));
    const { data, loading, error } = store.getState().bioEntity;

    expect(type).toBe('project/getBioEntityContents/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(bioEntityResponseFixture.content);
  });

  it('should update store after failed getBioEntity query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.NotFound, bioEntityResponseFixture);

    const { type } = await store.dispatch(getBioEntity(SEARCH_QUERY));
    const { data, loading, error } = store.getState().bioEntity;

    expect(type).toBe('project/getBioEntityContents/rejected');
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual([]);
  });

  it('should update store on loading getBioEntity query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

    const bioEntityContentsPromise = store.dispatch(getBioEntity(SEARCH_QUERY));

    const { data, loading } = store.getState().bioEntity;
    expect(data).toEqual([]);
    expect(loading).toEqual('pending');

    bioEntityContentsPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } = store.getState().bioEntity;

      expect(dataPromiseFulfilled).toEqual(bioEntityResponseFixture.content);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
